@@ -16,18 +16,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dynarray.cc 1.2.1.1.1.5.1.8.2.7 Tue, 29 Sep 1998 13:26:59 -0700 jmacd $
+ * $Id: dynarray.cc 1.2.1.1.1.5.1.8.2.7.1.1 Tue, 05 Feb 2002 18:58:12 +0300 jmacd $
  */
 
-
-#include <stdarg.h>
-#include "config.h"
-#ifdef HAVE_STD_H
-#include <std.h>
-#endif
-#include "dynarray.h"
-#include "dstring.h"
-#include "utils.h"
+#include "prcs.h"
 
 #define generic template<class T, int DefaultSize, bool ZeroTerm>
 #define selftype Dynarray<T, DefaultSize, ZeroTerm>
