@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcserror.h 1.14.1.5.1.11 Sun, 31 Oct 1999 07:26:02 +0300 jmacd $
+ * $Id: prcserror.h 1.14.1.5.1.11.2.1 Fri, 26 Apr 2002 18:24:52 +0400 jmacd $
  */
 
 
@@ -27,12 +27,17 @@
 #include <iostream.h>
 #include <iomanip.h>
 #if defined(__GNUG__)
-#include <stdiostream.h>
-#include <strstream.h>
+# if defined(__APPLE__)
+#  include <streambuf.h>
+   typedef filebuf stdiobuf;
+# else
+#  include <stdiostream.h>
+# endif /* if defined(__APPLE__) */
+# include <strstream.h>
 #else
-#include <fstream.h>
-typedef filebuf stdiobuf;
-#include "be-strstream.h"
+# include <fstream.h>
+  typedef filebuf stdiobuf;
+# include "be-strstream.h"
 #endif
 
 
