@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: utils.h 1.18.1.2 Mon, 01 Jun 1998 22:12:21 -0700 jmacd $
+ * $Id: utils.h 1.18.1.3 Wed, 24 Oct 2001 14:23:43 +0400 jmacd $
  */
 
 /* This file contains a bunch of stuff, including some really super ugly
@@ -44,7 +44,7 @@
  *     case you want to forget why you put an assertion in in the
  *     first place.  */
 #ifndef NDEBUG
-EXTERN void abort(void);
+/*EXTERN void abort(void);*/
 #include <stdio.h>
 #if defined(__GNUG__) && defined(PRCS_DEVEL)
 #define ASSERT(condition, message) \
