@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: misc.cc 1.30.1.3.1.17.1.11.1.7.1.18.1.33 Sun, 31 Oct 1999 16:52:32 -0800 jmacd $
+ * $Id: misc.cc 1.30.1.3.1.17.1.11.1.7.1.18.1.35 Thu, 07 Feb 2002 07:57:16 +0300 jmacd $
  */
 
 extern "C" {
@@ -82,6 +82,8 @@ int option_not_match_file = 0;
 int option_all_files = 0;
 int option_preorder = 0;
 int option_pipe = 0;
+int option_nokeywords = 0;
+int option_version_log = 0;
 #ifdef PRCS_DEVEL
 int option_n_debug = 1;
 int option_tune = 10;
@@ -91,20 +93,20 @@ int option_skilled_merge = 0;
 int option_plain_format = 0;
 int option_sort = 0;
 
-const char *option_match_file_pattern;
-const char *option_not_match_file_pattern;
-const char *option_sort_type;
-
+const char *option_match_file_pattern = NULL;
+const char *option_not_match_file_pattern = NULL;
+const char *option_sort_type = NULL;
+const char *option_version_log_string = NULL;
 
 const char* temp_file_1 = NULL;
 const char* temp_file_2 = NULL;
 const char* temp_file_3 = NULL;
 const char* temp_directory = NULL;
 
+const char* maintainer = "prcs-bugs@XCF.Berkeley.EDU";
+
 PrVoidError bug (void)
 {
-  static const char maintainer[] = "prcs-bugs@XCF.Berkeley.EDU";
-
   pthrow prcserror << "Please report this to " << maintainer << dotendl
 		  << "When sending bug reports, always include:" << prcsendl
 		  << "-- a complete description of the problem encountered" << prcsendl
@@ -1130,10 +1132,10 @@ NprVoidError Err_write(int fd, const void* buf, size_t nbytes)
 	do {
 	    c = write(fd, buf, nbytes);
 	} while (c < 0 && errno == EINTR);
-	
+
 	if (c < 0)
 	    return WriteFailure;
-	
+
 	nbytes -= c;
 	buf = ((const char*)buf) + c;
     }
