@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: prcserror.cc 1.8.1.11.1.3.1.7.1.11.2.1 Mon, 04 Feb 2002 00:39:38 +0300 jmacd $
+ * $Id: prcserror.cc 1.8.1.11.1.3.1.7.1.11.2.2 Fri, 26 Apr 2002 18:24:52 +0400 jmacd $
  */
 
 
@@ -48,8 +48,13 @@ ErrorToken global_error_token;
 int return_if_fail_if_ne_val;
 #endif
 
-stdiobuf stdout_stream(stdout);
-stdiobuf stderr_stream(stderr);
+#if defined(__APPLE__)
+ stdiobuf stdout_stream(STDOUT_FILENO);
+ stdiobuf stderr_stream(STDERR_FILENO);
+#else
+ stdiobuf stdout_stream(stdout);
+ stdiobuf stderr_stream(stderr);
+#endif /* if defined(__APPLE__) */
 strstreambuf query_stream;
 
 static PrettyStreambuf pretty_stdout_stream(&stdout_stream, NULL);
