@@ -16,13 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: projdesc.cc 1.9.1.1.1.4.1.17.1.5.1.17.3.9.1.32 Sat, 30 Oct 1999 21:26:02 -0700 jmacd $
+ * $Id: projdesc.cc 1.9.1.1.1.4.1.17.1.5.1.17.3.9.1.34.2.1 Tue, 05 Feb 2002 18:58:12 +0300 jmacd $
  */
 
 extern "C" {
 #include <stdarg.h>
 }
 
+#if __GNUG__ < 3
+#define pubsync sync
+#endif
+
 #include "prcs.h"
 #include "projdesc.h"
 #include "sexp.h"
@@ -109,12 +113,9 @@ ProjectDescriptor::~ProjectDescriptor()
     }
 
     if (_log_pstream) {
-#ifdef __GNUG__
-	if (_log_pstream->sync())
-#else
-        if (_log_pstream->pubsync())
-#endif
+        if (_log_pstream->pubsync()) {
 	    prcserror << "warning: Write to merge log failed" << perror;
+	}
 
 	delete _log_pstream;
 	delete _log_stream;
@@ -271,10 +272,19 @@ PrVoidError ProjectDescriptor::init_from_file(const char* full_name,
     _project_aux_path.sprintf("%s.%s.prcs_aux", _project_path.cast(), _project_name);
 
     if(infile0 == NULL) {
+	struct stat buf;
+
         if((infile = fopen(filename, "r")) == NULL)
 	    pthrow prcserror << "Can't open file " << squote(filename) << perror;
+
+	If_fail(Err_stat(filename, & buf))
+	    pthrow prcserror << "Stat failed on project file "
+			     << squote (filename) << perror;
+
+	_read_mode = buf.st_mode & 0777;
     } else {
 	infile = infile0;
+	_read_mode = 0;
     }
 
     _prj_source_name = filename;
@@ -342,13 +352,9 @@ PrVoidError ProjectDescriptor::init_merge_log()
 
     _log_stream = new filebuf;
 
-#ifndef __GNUG__
-#  define APPENDTAG ios::out|ios::app
-#else
-#  define APPENDTAG "a"
-#endif
-    if (!_log_stream->open(logname, APPENDTAG))
+    if (!_log_stream->open(logname, ios::out|ios::app)) {
 	pthrow prcserror << "Failed opening merge log file " << squote(logname) << perror;
+    }
 
     _log_pstream = new PrettyStreambuf (_log_stream, &option_report_actions);
 
@@ -1711,6 +1717,11 @@ PrVoidError ProjectDescriptor::write_project_file(const char* filename)
 
     Return_if_fail (outfile.close());
 
+    if (_read_mode != 0) {
+	If_fail (Err_chmod (filename, _read_mode))
+	    prcswarning << "Could not reset permissions on file " << squote (filename) << dotendl;
+    }
+
     return NoError;
 }
 
@@ -1837,18 +1848,11 @@ void ProjectDescriptor::really_write_project_file (ostream& os)
 		print_protected (project_name(), os);
 		os << "/";
 		print_protected (fe->descriptor_name(), os);
-#ifdef __GNUG__
-		os.form(" %s %03o",
-			fe->descriptor_version_number(),
-			fe->file_mode());
-#else
-                {
-                  char buf[8];
-                  sprintf(buf, "%03o", fe->file_mode());
-                  os << " " << fe->descriptor_version_number()
-                     << " " << buf;
-                }
-#endif
+
+		char buf[8];
+		sprintf(buf, "%03o", fe->file_mode());
+		os << " " << fe->descriptor_version_number()
+		   << " " << buf;
 	    }
 
 	    os << ")";
@@ -2175,8 +2179,7 @@ PrVoidError ProjectDescriptor::parse_prj_file(FILE* infile)
     prj_lex_cur_index = 0;
     prj_lex_this_index = 0;
 
-    If_fail (prj_seg->map_file (fileno(infile)))
-	pthrow prcserror << "Failed reading from file " << squote (_prj_source_name) << perror;
+    Return_if_fail (prj_seg->map_file (_prj_source_name, fileno(infile)));
 
     _segment->append_segment("", 1); /* Added a null here for safety, going to
 				      * have to get rid of it when writing. */
