@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: rebuild.cc 1.3.1.1.1.8.1.21.1.4.1.4.1.14.1.4 Thu, 07 Feb 2002 07:57:16 +0300 jmacd $
+ * $Id: rebuild.cc 1.3.1.1.1.8.1.21.1.4.1.4.1.14.1.5 Fri, 26 Apr 2002 17:41:07 +0400 jmacd $
  */
 
 extern "C" {
@@ -264,7 +264,7 @@ static PrVoidError delete_unreferenced_versions_file(RepEntry* rep_entry,
 static PrVoidError ignoring(ProjectVersionData* project_data)
 {
     prcswarning << "Project version " << project_data << "'s project file is invalid "
-      "and will be ignored. " << dotendl;
+      "and will be ignored" << dotendl;
 
     bug ();
 
